/*
 *
 * Copyright (C) 2000  Robert Olsson, Jens Ls 
 * Swedish University of Agricultural Sciences
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#include "pim_common.h"
ZBUG_EXTERN

char b1[16], b2[16], b3[16], b4[16];  /* For inet_2a */


int rt_1_wc_1_timeout (struct thread *t_entry_timeout)
{
  struct mgrp *mg = THREAD_ARG (t_entry_timeout);
  mg->pim.t_entry_timeout = NULL;

  ZBUG(ZBUG_PIM_TIMER)
    zlog_warn("***** rt_1_wc_1_(entry)timeout\n");

    printf("XXXX  mgrp timeout for %s %lu XXXX\n", 
	   inet_2a(mg->group, b1), mg->pim.exptime);
  
  if(mg->flags & MRT_IGMP_IN_USE) {
    mg->flags &= ~MRT_PIM_IN_USE;
    ZBUG_G(mg->group)
      fprintf(ZBUG_LOG, "PIM (*,G) entry timeout, IGMP left\n");
    
    return 0;
  }

  ZBUG_G(mg->group)
    fprintf(ZBUG_LOG, "PIM (*,G) entry timeout\n");
  
  /* Delete this entry 
     All SPT with RPT deleted too
     */

  mg_remove(mg);
  return 0;
}

int mg_oif_timeout (struct thread *t_oif_timeout)
{
  struct oif *mi = THREAD_ARG (t_oif_timeout);
  struct mgrp *mg;
  mi->pim.t_oif_timeout = NULL;
  mg = mi->mg;

  ZBUG(ZBUG_PIM_TIMER)
    zlog_warn("***** (*,G) oif timeout\n");
  
  ZBUG_G(mg->group)
    {
     fprintf(ZBUG_LOG, "(*,%s) oif timeout\n",
	     inet_2a(mg->group, b2));
     fflush(ZBUG_LOG); 
    }
  
  if(mi->flags & MRT_IGMP_IN_USE)
    {
      mi->flags &= ~MRT_PIM_IN_USE;
      return 0;
    }

//  VIFM_CLR(mi->ifp->ifindex, mg->oif_bitmap);
  mg_oif_list_delete(mi); /* struct mi deallocated here! */
  
  if(!list_isempty(mg->oif_list)) return 0;

  /* Trigger join/prune (this group will be in prune-list) */
  mg->flags |= MRT_PRUNED;
  ZBUG_G(mg->group)
    {
     fprintf(ZBUG_LOG, "(*,%s) PRUNED\n",
	     inet_2a(mg->group, b2));
      fflush(ZBUG_LOG); 
    }
  if(mg->pim.rp->ifp) /* mg->ifp points towards RP */
    trigger_join_prune(mg->pim.rp->ifp);
  return 0;
}


int mr_oif_timeout (struct thread *t_oif_timeout)
{
  struct mrt *mr;
  struct oif *mi = THREAD_ARG (t_oif_timeout);
  mi->pim.t_oif_timeout = NULL;

  mr = mi->mr;
  
  ZBUG(ZBUG_PIM_TIMER)
    {
      if(mr->flags & MRT_SPT) 
	zlog_warn("***** (S,G) oif timeout\n");
      else
	zlog_warn("***** pseudo (S,G) oif timeout\n");
    }
  ZBUG_G(mr->mg->group)
    {
      fprintf(ZBUG_LOG, "(%s,%s) oif %s timeout ",
	      inet_2a(mr->ms->src, b1),
	      inet_2a(mr->mg->group, b2),
	      mi->ifp->name);
     pim_flags_f(ZBUG_LOG, mr->ms->src, mr->mg->group, mi->flags);
     fflush(ZBUG_LOG); 
    }
  
  mr_oif_list_delete(mi);
  
  if(!list_isempty(mr->oif_list)) return 0;

  /* Trigger join/prune (this S,G will be in prune-list) */
  mr->flags |= MRT_PRUNED;

  ZBUG_G(mr->mg->group)
    {
     fprintf(ZBUG_LOG, "(%s,%s) PRUNED ",
	      inet_2a(mr->ms->src, b1),
	      inet_2a(mr->mg->group, b2));
     pim_flags_f(ZBUG_LOG, mr->ms->src, mr->mg->group, mi->flags);
     fflush(ZBUG_LOG); 
    }
  
  if(mr->flags & MRT_SPT)
    {
      /* send joinprune towards source */
      trigger_join_prune(mr->ms->ifp);
    }
  
  return 0;
}


void oif_timer_update(struct mrt *mr, struct interface *ifp, u_int32_t newtime)
{
  struct oif *mi;
  u_int32_t found;
  listnode node;

  found = 0;
  for (node = listhead (mr->oif_list); node; nextnode (node)) {
    mi = getdata (node);
    zassert(mi, -1);
    if(mi->ifp == ifp ) {
	found = 1;
	break;
    }
  }
  if( !found) return;

  mi->pim.oif_deletion_delay = newtime/3;
  
  if(mi->pim.exptime < time(NULL) + newtime) {
    if( mi->pim.t_oif_timeout) thread_cancel(mi->pim.t_oif_timeout);
    mi->pim.exptime = time(NULL)+newtime;
    mi->pim.oif_deletion_delay = newtime/3;
    mi->pim.t_oif_timeout = thread_add_timer(master, mr_oif_timeout, mi, 
					 newtime);
  }
}

void update_mr_oif_timers(struct mrt *mr, struct interface *ifp, u_int16_t newtime)
{
  struct oif *mi;
  listnode node;

  for (node = listhead (mr->oif_list); node; nextnode (node)) {
    mi = getdata (node);
    zassert(mi, -1);
    if(mi->ifp == ifp )
      {
	mi->flags |= MRT_PIM_IN_USE;
	
	if(mi->pim.exptime < time(NULL) + newtime) {

	  if( mi->pim.t_oif_timeout) thread_cancel(mi->pim.t_oif_timeout);
	  mi->pim.exptime = time(NULL)+newtime;
	  mi->pim.oif_deletion_delay = newtime/3;
	  mi->pim.t_oif_timeout = thread_add_timer(master, mr_oif_timeout, mi, 
					       newtime);
	}
	else   printf("Ignoring oif timer setting\n");
	return;
      }
  }
}
